#include <pcl/io/pcd_io.h>
#include <pcl/filters/passthrough.h>
#include <pcl/filters/voxel_grid.h>
#include <pcl/visualization/pcl_visualizer.h>

int main(int argc, char **argv) 
{
  pcl::PointCloud<pcl::PointXYZ>::Ptr in_cloud(new pcl::PointCloud<pcl::PointXYZ> ()),
                                      cutted_cloud(new pcl::PointCloud<pcl::PointXYZ> ()),
                                      subsamp_cloud(new pcl::PointCloud<pcl::PointXYZ> ());
  
  if ( pcl::io::loadPCDFile( "office_scene.pcd", *in_cloud ) < 0)
  {
    std::cout << "Error loading model cloud." << std::endl;
    return (-1);
  }
  
  boost::shared_ptr<pcl::visualization::PCLVisualizer> viewer (new pcl::visualization::PCLVisualizer ("3D Viewer"));
  viewer->setBackgroundColor (0, 0, 0);
  viewer->addPointCloud<pcl::PointXYZ> ( in_cloud, "Input cloud" );
  viewer->addCoordinateSystem (1.0);
  viewer->initCameraParameters ();
  
  while (!viewer->wasStopped ())
  {
    viewer->spinOnce ( 1 );
  }
  
  pcl::PassThrough<pcl::PointXYZ> pass_through;
  pass_through.setInputCloud (in_cloud);
  pass_through.setFilterLimits (0.0, 2.0);
  pass_through.setFilterFieldName ("y");
  pass_through.filter( *cutted_cloud );
  
  pcl::visualization::PointCloudColorHandlerCustom<pcl::PointXYZ> cutted_color(cutted_cloud, 0, 255, 0);
  viewer->addPointCloud<pcl::PointXYZ> ( cutted_cloud, cutted_color, "Cutted cloud" );
  
  viewer->resetStoppedFlag();
  while (!viewer->wasStopped ())
  {
    viewer->spinOnce ( 1 );
  }
  
  viewer->removePointCloud("Input cloud");
  viewer->resetStoppedFlag();
  while (!viewer->wasStopped ())
  {
    viewer->spinOnce ( 1 );
  }
  
  pcl::VoxelGrid<pcl::PointXYZ> voxel_grid;
  voxel_grid.setInputCloud (cutted_cloud);
  voxel_grid.setLeafSize (0.1, 0.1, 0.1);
  voxel_grid.filter ( *subsamp_cloud ) ;
  
  pcl::visualization::PointCloudColorHandlerCustom<pcl::PointXYZ> subsamp_color(subsamp_cloud, 255, 0, 0);
  viewer->addPointCloud<pcl::PointXYZ> ( subsamp_cloud, subsamp_color, "Voxelized cloud" );
  viewer->setPointCloudRenderingProperties (pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 3, "Voxelized cloud");
  
  viewer->resetStoppedFlag();
  while (!viewer->wasStopped ())
  {
    viewer->spinOnce ( 1 );
  }
  
  viewer->removePointCloud("Cutted cloud");
  viewer->resetStoppedFlag();
  while (!viewer->wasStopped ())
  {
    viewer->spinOnce ( 1 );
  }
  
  if ( pcl::io::savePCDFile( "res_cloud.pcd", *subsamp_cloud) < 0)
  {
    std::cout << "Error saving model cloud." << std::endl;
    return (-1);
  }
}
